=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-03-03 03:09:07 +0000
+++ bzrlib/repository.py	2006-03-03 07:00:50 +0000
@@ -93,7 +93,7 @@
         """Construct the current default format repository in a_bzrdir."""
         return RepositoryFormat.get_default_format().initialize(a_bzrdir)
 
-    def __init__(self, _format, a_bzrdir, control_files, revision_store, text_store):
+    def __init__(self, _format, a_bzrdir, control_files, revision_store, control_store, text_store):
         """instantiate a Repository.
 
         :param _format: The format of the repository on disk.
@@ -114,6 +114,10 @@
         self.text_store = text_store
         # backwards compatability
         self.weave_store = text_store
+        # not right yet - should be more semantically clear ? 
+        # 
+        self.control_store = control_store
+        self.control_weaves = control_store
 
     def lock_write(self):
         self.control_files.lock_write()
@@ -204,28 +208,10 @@
     @needs_read_lock
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
-        xml_file = self.get_revision_xml_file(revision_id)
-
-        try:
-            r = bzrlib.xml5.serializer_v5.read_revision(xml_file)
-        except SyntaxError, e:
-            raise bzrlib.errors.BzrError('failed to unpack revision_xml',
-                                         [revision_id,
-                                          str(e)])
-            
-        assert r.revision_id == revision_id
-        return r
-
-    @needs_read_lock
-    def get_revision_sha1(self, revision_id):
-        """Hash the stored value of a revision, and return it."""
-        # In the future, revision entries will be signed. At that
-        # point, it is probably best *not* to include the signature
-        # in the revision hash. Because that lets you re-sign
-        # the revision, (add signatures/remove signatures) and still
-        # have all hash pointers stay consistent.
-        # But for now, just hash the contents.
-        return bzrlib.osutils.sha_file(self.get_revision_xml_file(revision_id))
+        if not revision_id or not isinstance(revision_id, basestring):
+            raise InvalidRevisionId(revision_id=revision_id, branch=self)
+        return self._revision_store.get_revision(revision_id,
+                                                 self.get_transaction())
 
     @needs_write_lock
     def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
@@ -467,7 +453,7 @@
 class AllInOneRepository(Repository):
     """Legacy support - the repository behaviour for all-in-one branches."""
 
-    def __init__(self, _format, a_bzrdir, revision_store, text_store):
+    def __init__(self, _format, a_bzrdir, revision_store, control_store, text_store):
         # we reuse one control files instance.
         dir_mode = a_bzrdir._control_files._dir_mode
         file_mode = a_bzrdir._control_files._file_mode
@@ -505,26 +491,22 @@
         # not broken out yet because the controlweaves|inventory_store
         # and text_store | weave_store bits are still different.
         if isinstance(_format, RepositoryFormat4):
+            # cannot remove these - there is still no consistent api 
+            # which allows access to this old info.
             self.inventory_store = get_store('inventory-store')
             text_store = get_store('text-store')
-        elif isinstance(_format, RepositoryFormat5):
-            self.control_weaves = get_weave('')
-        elif isinstance(_format, RepositoryFormat6):
-            self.control_weaves = get_weave('')
-        else:
-            raise errors.BzrError('unreachable code: unexpected repository'
-                                  ' format.')
-        super(AllInOneRepository, self).__init__(_format, a_bzrdir, a_bzrdir._control_files, revision_store, text_store)
+        super(AllInOneRepository, self).__init__(_format, a_bzrdir, a_bzrdir._control_files, revision_store, control_store, text_store)
 
 
 class MetaDirRepository(Repository):
     """Repositories in the new meta-dir layout."""
 
-    def __init__(self, _format, a_bzrdir, control_files, revision_store, text_store):
+    def __init__(self, _format, a_bzrdir, control_files, revision_store, control_store, text_store):
         super(MetaDirRepository, self).__init__(_format,
                                                 a_bzrdir,
                                                 control_files,
                                                 revision_store,
+                                                control_store,
                                                 text_store)
 
         dir_mode = self.control_files._dir_mode
@@ -544,14 +526,6 @@
                 ws.enable_cache = True
             return ws
 
-        if isinstance(self._format, RepositoryFormat7):
-            self.control_weaves = get_weave('')
-        elif isinstance(self._format, RepositoryFormatKnit1):
-            self.control_weaves = get_weave('control')
-        else:
-            raise errors.BzrError('unreachable code: unexpected repository'
-                                  ' format.')
-
 
 class RepositoryFormat(object):
     """A repository format.
@@ -595,6 +569,10 @@
         except KeyError:
             raise errors.UnknownFormatError(format_string)
 
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        raise NotImplementedError(self._get_control_store)
+    
     @classmethod
     def get_default_format(klass):
         """Return the current default format."""
@@ -732,6 +710,13 @@
             control_files.unlock()
         return self.open(a_bzrdir, _found=True)
 
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return self._get_versioned_file_store('',
+                                              repo_transport,
+                                              control_files,
+                                              prefixed=False)
+
     def _get_text_store(self, transport, control_files):
         """Get a store for file texts for this format."""
         raise NotImplementedError(self._get_text_store)
@@ -744,11 +729,13 @@
 
         repo_transport = a_bzrdir.get_repository_transport(None)
         control_files = a_bzrdir._control_files
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
         revision_store = self._get_revision_store(repo_transport, control_files)
-        text_store = self._get_text_store(repo_transport, control_files)
         return AllInOneRepository(_format=self,
                                   a_bzrdir=a_bzrdir,
                                   revision_store=revision_store,
+                                  control_store=control_store,
                                   text_store=text_store)
 
 
@@ -781,6 +768,13 @@
         """
         return False
 
+    def _get_control_store(self, repo_transport, control_files):
+        """Format 4 repositories have no formal control store at this point.
+        
+        This will cause any control-file-needing apis to fail - this is desired.
+        """
+        return None
+    
     def _get_revision_store(self, repo_transport, control_files):
         """See RepositoryFormat._get_revision_store()."""
         return self._get_text_rev_store(repo_transport,
@@ -860,6 +854,13 @@
         control_files = LockableFiles(repository_transport, 'lock')
         return control_files
 
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return self._get_versioned_file_store('',
+                                              repo_transport,
+                                              control_files,
+                                              prefixed=False)
+
     def _get_revision_store(self, repo_transport, control_files):
         """See RepositoryFormat._get_revision_store()."""
         return self._get_text_rev_store(repo_transport,
@@ -884,12 +885,14 @@
         else:
             repo_transport = a_bzrdir.get_repository_transport(None)
         control_files = LockableFiles(repo_transport, 'lock')
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
         revision_store = self._get_revision_store(repo_transport, control_files)
-        text_store = self._get_text_store(repo_transport, control_files)
         return MetaDirRepository(_format=self,
                                  a_bzrdir=a_bzrdir,
                                  control_files=control_files,
                                  revision_store=revision_store,
+                                 control_store=control_store,
                                  text_store=text_store)
 
     def _upload_blank_content(self, a_bzrdir, dirs, files, utf8_files, shared):

=== modified file 'bzrlib/store/revision/__init__.py'
--- bzrlib/store/revision/__init__.py	2006-03-03 03:09:07 +0000
+++ bzrlib/store/revision/__init__.py	2006-03-03 07:00:50 +0000
@@ -22,9 +22,15 @@
 
 
 from copy import deepcopy
+from cStringIO import StringIO
 from unittest import TestSuite
 
 
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.trace import mutter
+
+
 class RevisionStoreTestProviderAdapter(object):
     """A tool to generate a suite testing multiple repository stores.
 
@@ -66,7 +72,27 @@
 
 
 class RevisionStore(object):
-    """A revision store stores revisions and signatures."""
+    """A revision store stores revisions."""
+
+    def add_revision(self, revision, transaction):
+        """Add revision to the revision store.
+
+        :param rev: The revision object.
+        """
+        # serialisation : common to all at the moment.
+        rev_tmp = StringIO()
+        bzrlib.xml5.serializer_v5.write_revision(revision, rev_tmp)
+        rev_tmp.seek(0)
+        self._add_revision(revision, rev_tmp, transaction)
+        mutter('added revision_id {%s}', revision.revision_id)
+
+    def _add_revision(self, revision, revision_as_file, transaction):
+        """Template method helper to store revision in this store."""
+        raise NotImplementedError(self._add_revision)
+
+    def get_revision(self, revision_id, transaction):
+        """Return the Revision object for a named revision."""
+        raise NotImplementedError(self.get_revision)
 
     def has_revision_id(self, revision_id, transaction):
         """True if the store contains revision_id."""

=== modified file 'bzrlib/store/revision/knit.py'
--- bzrlib/store/revision/knit.py	2006-03-03 03:09:07 +0000
+++ bzrlib/store/revision/knit.py	2006-03-03 07:00:50 +0000
@@ -20,6 +20,8 @@
 """
 
 
+import bzrlib
+import bzrlib.errors as errors
 from bzrlib.store.revision import RevisionStore
 from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.transport import get_transport
@@ -48,8 +50,34 @@
         :param versioned_file_store: the text store to use for storing 
                                      revisions and signatures.
         """
+        super(KnitRevisionStore, self).__init__()
         self.versioned_file_store = versioned_file_store
 
+    def _add_revision(self, revision, revision_as_file, transaction):
+        """Template method helper to store revision in this store."""
+        self.get_revision_file(transaction).add_lines(
+            revision.revision_id,
+            revision.parent_ids,
+            revision_as_file.readlines())
+
+    def get_revision(self, revision_id, transaction):
+        """See RevisionStore.get_revision()."""
+        xml = self._get_revision_xml(revision_id, transaction)
+        try:
+            r = bzrlib.xml5.serializer_v5.read_revision_from_string(xml)
+        except SyntaxError, e:
+            raise errors.BzrError('failed to unpack revision_xml',
+                                   [revision_id,
+                                   str(e)])
+        assert r.revision_id == revision_id
+        return r
+
+    def _get_revision_xml(self, revision_id, transaction):
+        try:
+            return self.get_revision_file(transaction).get_text(revision_id)
+        except (errors.RevisionNotPresent):
+            raise errors.NoSuchRevision(self, revision_id)
+
     def get_revision_file(self, transaction):
         """Get the revision versioned file object."""
         return self.versioned_file_store.get_weave('revisions', transaction)

=== modified file 'bzrlib/store/revision/text.py'
--- bzrlib/store/revision/text.py	2006-03-03 03:09:07 +0000
+++ bzrlib/store/revision/text.py	2006-03-03 07:00:50 +0000
@@ -20,8 +20,11 @@
 """
 
 
+import bzrlib
+import bzrlib.errors as errors
 from bzrlib.store.revision import RevisionStore
 from bzrlib.store.text import TextStore
+from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.transport import get_transport
 
 
@@ -50,8 +53,32 @@
 
         :param text_store: the text store to put serialised revisions into.
         """
+        super(TextRevisionStore, self).__init__()
         self.text_store = text_store
 
+    def _add_revision(self, revision, revision_as_file, transaction):
+        """Template method helper to store revision in this store."""
+        self.text_store.add(revision_as_file, revision.revision_id)
+
+    def get_revision(self, revision_id, transaction):
+        """See RevisionStore.get_revision()."""
+        xml_file = self._get_revision_xml_file(revision_id)
+        try:
+            r = bzrlib.xml5.serializer_v5.read_revision(xml_file)
+        except SyntaxError, e:
+            raise errors.BzrError('failed to unpack revision_xml',
+                                   [revision_id,
+                                   str(e)])
+            
+        assert r.revision_id == revision_id
+        return r
+
+    def _get_revision_xml_file(self, revision_id):
+        try:
+            return self.text_store.get(revision_id)
+        except (IndexError, KeyError):
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
+
     def has_revision_id(self, revision_id, transaction):
         """True if the store contains revision_id."""
         return (revision_id is None

=== modified file 'bzrlib/tests/revisionstore_implementations/test_all.py'
--- bzrlib/tests/revisionstore_implementations/test_all.py	2006-03-03 03:09:07 +0000
+++ bzrlib/tests/revisionstore_implementations/test_all.py	2006-03-03 07:00:50 +0000
@@ -39,19 +39,17 @@
         self.store = self.store_factory.create(self.get_url('.'))
         self.transaction = PassThroughTransaction()
 
-    def test_add(self):
-        # adding a revision should fail if the inventory is not present.
-        inv = EmptyTree().inventory
-        sha1 = "111111111111111111111111111111111111111"
+    def test_add_has_get(self):
         rev = Revision(timestamp=0,
                        timezone=None,
                        committer="Foo Bar <foo@example.com>",
                        message="Message",
-                       inventory_sha1=sha1,
-                       revision_id='should_fail')
-        self.assertRaises(errors.InventoryNotPresent,
-                          self.store.add_revision,
-                          rev, self.transaction)
+                       inventory_sha1='aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
+                       revision_id='A')
+        self.store.add_revision(rev, self.transaction)
+        self.assertTrue(self.store.has_revision_id('A', self.transaction))
+        rev2 = self.store.get_revision('A', self.transaction)
+        self.assertEqual(rev, rev2)
 
     def test_has_missing(self):
         # has of a non present id -> False
@@ -60,3 +58,10 @@
     def test_has_None(self):
         # has of None -> True
         self.assertTrue(self.store.has_revision_id(None, self.transaction))
+
+    def test_get_revision_none(self):
+        # get_revision(None) -> raises NoSuchRevision
+        self.assertRaises(errors.NoSuchRevision,
+                          self.store.get_revision,
+                          'B',
+                          self.transaction)

